@@ -11,7 +11,7 @@ def get_dummy_info_supplier(history):
 
 class ForestGrower:
     
-    def __init__(self, info_supplier, d, step_size, w_min, epsilon, delta, extrapolation_multiplier, max_trees, random_state, stop_when_horizontal, logger):
+    def __init__(self, info_supplier, d, step_size, w_min, epsilon, delta, extrapolation_multiplier, max_trees, random_state, stop_when_horizontal, bootstrap_repeats, logger):
         
         if w_min <= step_size:
             raise ValueError("\"w_min\" must be strictly bigger than the value of \"step_size\".")
@@ -27,6 +27,8 @@ class ForestGrower:
         self.logger = logger
         self.random_state = random_state
         self.stop_when_horizontal = stop_when_horizontal
+        self.bootstrap_repeats = bootstrap_repeats
+        self.bootstrap_init_seed = self.random_state.randint(10**5)
         
     def estimate_slope(self, window:np.ndarray):
         max_index_to_have_two_values = max(0, len(window) - self.w_min)
@@ -36,6 +38,7 @@ class ForestGrower:
         window_domain = np.array(range(0, len(window) * self.step_size + 1, self.step_size))
         self.logger.debug(f"\tEstimating slope for window of size {len(window)}.")
         def get_slope(indices = slice(0, len(window))):
+            self.logger.debug(indices)
             if len(np.unique(indices)) == 1: # if there is just one value in the bootstrap sample, return nan
                 return np.nan
             cov = np.cov(np.array([window_domain[indices], window[indices]]))
@@ -44,9 +47,13 @@ class ForestGrower:
         
         if min(window) < max(window):
             try:
-                result = bootstrap((list(range(len(window))),), get_slope, vectorized = False, n_resamples = 2, random_state = self.random_state, method = "percentile")
-                ci = result.confidence_interval
-                return max(np.abs([ci.high, ci.low]))
+                if self.bootstrap_repeats > 1:
+                    result = bootstrap((list(range(len(window))),), get_slope, vectorized = False, n_resamples = self.bootstrap_repeats, random_state = self.bootstrap_init_seed + self.t, method = "percentile")
+                    ci = result.confidence_interval
+                    return max(np.abs([ci.high, ci.low]))
+                else:
+                    return np.abs(get_slope(list(range(len(window)))))
+                    
             except ValueError:
                 return 0
         else:
@@ -120,7 +127,6 @@ class ForestGrower:
                     self.start_of_convergence_window[i] = cur_window_start
                 w = len(history) - cur_window_start
                 self.is_cauchy[i] = w >= self.w_min
-                #print(self.t, self.start_of_convergence_window[i], w, self.is_cauchy[i], np.max(history[-self.w_min:]) - np.min(history[self.start_of_convergence_window[i]:]))
 
             # if the dimension is Cauchy convergent, also estimate the slope
             if self.is_cauchy[i]:
@@ -133,6 +139,8 @@ class ForestGrower:
                     raise Exception()
 
                 slope = self.estimate_slope(window)
+                if np.isnan(slope):
+                    slope = (max(window) - min(window)) / (len(window) - 1)
                 self.logger.debug(f"\tEstimated slope is {slope}. Maximum deviation on {self.extrapolation_multiplier} trees is {np.round(slope * self.extrapolation_multiplier, 4)}.")
                 self.slope_hist.append(slope)
                 if np.abs(slope * self.extrapolation_multiplier) < self.epsilon:
